module Frigate
	module Form
		# Uses to define a form association with properties as Frigate::Form::Property
		class Association
			class << self
				# defines property for association
				# @param [Symbol] name
				# @param [Hash] options
				def property(name, options={})
					@properties ||= {}
					@properties[name.to_sym] = options
				end

				# defines association (kinda property with properties) for association
				# @param [Symbol] name
				# @param [Hash] options
				# @param [Proc] block
				def has_one(name, options={}, &block)
					@associations ||= {}
					@associations[name.to_sym] = options.merge({ block: block })
				end

				# gets just options for defined properties
				# @example properties hash
				#    { skype: { validates: { presence: true }, another_option: another_option_hash } }
				def properties
					@properties.deep_dup || {}
				end

				# gets just options for defined associations
				# @example associations hash
				#     { some_name: { block: <Proc.new instance> }
				def associations
					@associations.deep_dup || {}
				end
			end

			attr_reader :name, :opts, :properties, :associations, :root, :parent
			# @param [Symbol] name
			# @param [Hash] opts
			def initialize(name, root, parent, opts = {})
				@name, @root, @parent, @opts = name, root, parent, opts
				@properties, @associations = [], []

				exec_opts_block
				process_properties
				process_associations
			end

			# gets errors of association
			# @return [ActiveModel::Errors]
			def errors
				@errors ||= ActiveModel::Errors.new(self)
			end

			# checks validness of association properties
			def valid?
				errors.messages.empty?
			end

			# validates association properties
			def validate
				properties.each { |_prop| errors.add(_prop.name, _prop.errors[:value]) unless _prop.valid? }
			end

			private

			# exec_opts_block
			def exec_opts_block
				singleton_class.instance_exec &@opts.delete(:block)
			end

			# initializes declared associations
			def process_associations
				singleton_class.associations.each do |_assoc_name, _assoc_options|
					_assoc = Association.new(_assoc_name, root, self, _assoc_options)
					@associations << _assoc
					define_singleton_method(_assoc.name) { _assoc }
				end
			end

			# initializes declared properties
			def process_properties
				singleton_class.properties.each do |_prop_name, _prop_options|
					_prop = Property.new(_prop_name, root, self, _prop_options)
					@properties << _prop
					define_singleton_method(_prop.name) { _prop }
				end
			end

			# TODO add am::verifications
		end
	end
end