module Frigate
	module Form
		# Uses to define a form property
		class Property
			attr_accessor :name, :options, :value, :root, :parent

			include ActiveModel::Validations

			# @param [Symbol] name
			# @param [Hash] options
			def initialize(name, root, parent, options)
				@name, @root, @parent, @options = name, root, parent, options

				set_validations
				set_custom_validations
			end

			private

			def set_validations
				validations = options.delete(:validates)
				singleton_class.class_eval { validates :value, validations } if validations
				true
			end

			def set_custom_validations
				validations = options.delete(:validate)
				validations.is_a?(Array) and validations.each do |validation|
					singleton_class.class_eval { validate &validation }
				end
			end

			def add_error(name)
				errors.add(:value, name)
			end

			alias_method :error, :add_error
		end
	end
end