module Frigate
	module Form
		# Serves for synchronization form properties with model attributes/properties
		module Synchronizer
			# A syncronizator between form properties and model properties
			class Basic < Base
				# A class for synchronization model with params
				class ModelSynchronizer < Fundamental
					# Synchronizes given model with form properties and associations:properties
					# @param [Hash] ops
					def sync_with_model(opts={})
						sync_properties
						sync_associations
						save_model if opts[:save]
					end

					private

					def save_model
						form.model.save!
					end

					def sync_associations
						_sync_associations(form, form.model)
					end

					def sync_properties
						_sync_properties(form, form.model)
					end

					def _sync_associations(f_ctx, m_ctx)
						f_ctx.associations.each do |_assoc|
							n_m_ctx = m_ctx.send(_assoc.name.to_sym)
							if m_ctx.is_a?(ActiveRecord::Base) and n_m_ctx.nil?
								m_ctx.send("build_#{_assoc.name}")
							end
							_assoc.properties.each do |_prop|
								_sync_properties(_assoc, n_m_ctx)
							end
							_sync_associations(_assoc, n_m_ctx)
						end
					end

					def _sync_properties(f_ctx, m_ctx)
						f_ctx.properties.each do |_prop|
							m_ctx.send("#{_prop.name}=", _prop.value) unless _prop.value.nil?
						end
					end
				end

				# Synchronizes given params with given model
				# @param [Hash] opts
				# @return [Boolean] true
				def sync_with_model(opts={})
					ModelSynchronizer.new(form).sync_with_model(opts)
				end

				# Synchronizes properties errors with form errors
				# @return [Boolean]
				def sync_errors
					form.properties.each { |_prop| form.errors.add(_prop.name, _prop.errors[:value]) unless _prop.valid? }
					form.associations.each do |_assoc|
						_assoc.properties.each do |_prop|
							form.errors.add("#{_assoc.name}.#{_prop.name}", _prop.errors[:value]) unless _prop.valid?
						end
					end
				end

				def sync_properties_with_model_or_params
					raise NotImplementedError
				end
			end
		end
	end
end