module Frigate
	module Operation
		module Action
			# Base class is an abstact class
			# @abstract
			class Base
				class << self
					# Initializes action class and pass args to instance run method
					# @param [Frigate::Operation] operation receives an instace of subclass of Frigate::Operation
					# @param [Hash] params
					# @param [Hash] opts
					def run(operation, params, opts={})
						new(operation, opts).run(params)
					end
				end

				attr_reader :opts, :operation, :model, :model_form, :action_result, :params

				# @param [Frigate::Operation] operation
				# @param [Hash] opts
				def initialize(operation, opts={})
					@operation = operation
					@opts = opts
				end

				# Runs action's mechanism
				def run(params)
					(@params = params); action; model_form
				end

				# You should define an action in here
				def action
					raise NotImplementedError
				end

				protected

				# Gets a model class of defined model class for operation
				def model_klass
					operation.class.model_klass
				end

				# Creates an instance of operation subclass of Frigate::Form for given model
				def create_form(model)
					operation.class.form_class.new(model)
				end

				# A getter/caller of operation_block from an operation class variable *operation_block*
				# @note if no args passed then return the value of class variable *operation_block*
				# @note if args passed then class variable *operation_block* is going to be called with these args
				def operation_block(*args)
					if args.empty?
						operation.class.operation_block
					else
						if operation_block.nil?
							raise StandardError.new('operation_block is nil')
						else
							operation.class.operation_block.call(*args)
						end
					end
				end
			end
		end
	end
end