module Frigate
	module Operation
		# Is here initialize the whole operation class
		# A bigger description will be available soon
		class Base
			ALLOWED_ACTIONS = [:create, :update]

			class << self
				# Initializes form class and serves as a getter method
				def form_class
					@form_class ||= Class.new(Frigate::Form::Base)
				end

				# Serves as a getter method for *model_klass* class variable
				def model_klass
					@model_klass ? @model_klass : (raise StandardError.new('@model_klass nil'))
				end

				# Serves as a getter method for *operation_action* class variable
				def operation_action
					@operation_action ? @operation_action : (raise StandardError.new('@operation_action nil'))
				end

				# Serves as a getter method for *operation_block* class variable
				def operation_block
					# @operation_block.is_a?(Proc) ? @operation_block : (raise StandardError.new('operation block is not a Proc'))
					@operation_block
				end

				# @param [Class] model_klass
				def model(model_klass)
					@model_klass = model_klass
				end

				# @param [Symbol] action
				def action(action)
					ALLOWED_ACTIONS.include?(action) ? (@operation_action = action.to_sym) : (raise ArgumentError.new('action is invalid'))
				end

				# @param [Proc] block
				def operation(&block)
					@operation_block = block
				end

				delegate :property, :has_one, to: :form_class

				# @param [Hash] params
				# @param [Hash] opts
				def run(params, opts={})
					new(opts).run(params)
				end
			end

			attr_reader :params, :opts, :model_form

			delegate :errors, :model, to: :model_form

			# @param [Hash] opts
			def initialize(opts={})
				@opts = opts
			end

			# Runs the whole operation mechanism
			# @param [Hash] params
			# @return [Frigate::Operations] returns self
			def run(params)
				@params = Hashie::Mash.new(params)
				# running actions
				case self.class.operation_action
				when :update
					@model_form = Action::Update.run(self, params)
				else
					@model_form = Action::Create.run(self, params)
				end
				self # return self
			end
		end
	end
end