require 'active_support/concern'
module Frigate
	module Operation
		# Some stuff for controllers
		module Controller
			# Helpers for rails controllers
			module Helpers
				extend ActiveSupport::Concern

				included do
					rescue_from InvalidParamsError, with: :invalid_params_error

					def invalid_params_error(exception)
						respond_to do |f|
							f.json do
								render status: 400, json: {
										status: 'error',
										data: exception.errors.messages
									}
							end
						end
					end
				end

				# run-operation helps to run operation elegantly
				# @param [Frigate::Operation] klass
				# @param [Hash] params
				def run_operation(klass, params)
					klass.run(params)
				end

				# render-operation helps to render operation data
				# @param [Frigate::Operation] klass
				# @param [Hash] params
				def render_operation(*args)
					run_operation(*args).render(self)
					# raise run_operation(*args).errors.messages.inspect
				end
			end
		end
	end
end