module Frigate
	module Operation
		# allows operation to be rendered as json for now))
		module Renderable
			def self.included(base)
				base.extend(ClassMethods)
			end

			module ClassMethods
				def render_block
					@render_block
				end

				def render(render_block)
					if render_block.is_a?(Proc)
						@render_block ||= render_block
					else
						raise StandardError.new('render_block is not a Proc')
					end
				end
			end

			# @param [ActionController::Base] ctrl An Instance of subclass of ActionController::Base
			def render(ctrl)
				ctrl.respond_to do |f|
					f.json do
						ctrl.render status: (model_form.valid? ? 200 : 400), json: build_json(ctrl)
					end
				end
			end

			private

			# @param [ActionController::Base] ctrl An Instance of subclass of ActionController::Base
			def build_json(ctrl)
				if model_form.valid?
					Jbuilder.new { |_j| _j.status 'success'; _j.data { render_op(_j, self, ctrl) } }.target!
				else
					Jbuilder.new do |_j|
						_j.status 'error'
						_j.errors errors.messages
					end.target!
				end

			end

			def render_op(*args)
				self.class.render_block.call(*args) if self.class.render_block.is_a?(Proc)
			end
		end
	end
end