module Frigate
	module Operation
# A module that allows operation to perform as a background job, using Sidekiq as BJ for now
		module Worker
			# Included hook
			def self.included(base)
				base.send(:include, Sidekiq::Worker) # TODO: this will work with any background gem.
				base.extend(ClassMethods)
			end

			# Class methods of class that includes *Worker* module
			module ClassMethods
				# Overrides standard *run* method in order to run an operation as a background job
				# @param [Hash] params
				# @param [Hash] options
				def run(params, options = {})
					serialized_params = serialize_params(params)
					is_background? ? perform_async(serialized_params) : new.perform(serialized_params)
				end

				# Whether or not operation is background
				# @return [Boolean] either background or not
				def is_background?
					!(Rails.env.test? || Rails.env.cucumber?) ||
						(!ENV['FRIGATE_WORKER'].nil? || ENV['FRIGATE_WORKER'].downcase == 'true')
				end

				private
				# serializes params
				# @param [Hash] params
				def serialize_params(params)
					params # TODO: serialize AR for e.g.
				end
			end

			# Method that runs operation mechanism. It's here because of Sidekiq::Worker needs it
			# @param [Hash] params income parameters
			def perform(params)
				run(deserialize_params(params))
			end

			private
			# deserializes params
			# @param [Hash] params
			def deserialize_params(params)
				Hashie::Mash.new(params) # TODO: deserialize AR for e.g.
			end
		end
	end
end